# This module hold the Ruby implementations of the Flex display objects
# used by the framework.
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.

require 'conf'

module Flex
  
  # Base is the base class of all Ruby implementations of the Flex display objects.
  class Base
      attr_accessor :name, :id
      
      def initialize(name, id)
        raise ArgumentError if name == nil
        raise ArgumentError if id == nil
        @name = name
        @id = id
      end
      
      # This method returns either a new display object or returns the property of
      # a object. If the name argument is supplied it searches for a display object, 
      # if not it returns a property
      def method_missing(method_name, name=nil)
        #-- Checks whether the Flex application is synchronized
        is_sync
        if(name != nil)
          #-- Checks if the component exists
          value = check_component(name)
          if(value != false)
            #-- If it exists, it returns a new object
            Flex.const_get(shift_case(method_name.to_s)).new(name, value)
          else
            nil
          end
        else
          property = flex_object.checkProperty(@id.dup, shift_case(method_name.to_s,true))
          if(property != nil) 
            property
          else raise "No such property or method <#{method_name}> of class: #{self.class}"
          end
        end
      end
      
      # This method checks with the FunFX Flex adapter if the component exists. 
      # The name is the name of the display object.
      def check_component(name)
        count = $check_component
        while(!ready && count > 0)
          sleep(1)
          count = count -1
        end
        
        begin
          return flex_object.checkComponent(@id, name)
        rescue
          $stderr.print "IO failed: " + $!
          false
        end
      end
      
      # This method checks wheter the Flex application is ready, this means
      # if the framework is able to send requests
      def ready
        begin
          return flex_object.ready("FUNFX")
        rescue
          return false
        end
      end
      
      # Gets the Flas object from the Funfx instance
      def flex_object
        funfx.get_flex_object
      end
      
      # This method returns an instance of the Funfx class
      def funfx
        Funfx.instance
      end
      
      # This method lists all children of a display object
      def children
        flex_object.getChildren(@id)
      end
      
      # This method checks if the Flex application is synchronized or if it is working on some task
      def is_sync
        while(!flex_object.isSynchronized(nil))
          sleep($wait_for_synch)
        end
      end
      
      # This method makes it possible to both write syntax like the AutomationGenericEnv.xml, or writing
      # tests with Ruby syntax.
      # 
      # @ie.Button("button") is AutomationGenericEnv.xml syntax
      # @ie.button("button") is Ruby Syntax
      # 
      # Both is possible
      def shift_case(str, property=false)
        if(!property)
          str.gsub(/^[a-z]|[_][a-z]/) {|a| a.upcase}.delete("_")
        else
          str.gsub(/^[a-z]|[_][a-z]/) { |a| a.upcase}.delete("_").gsub(/^[A-Z]/){|a| a.downcase}
        end
      end
      
      # This method makes it possible to dynamically add a method to the class
      # This method is used when generating all classes at runtime with the XmlParser
      def self.add_method(name, &block )
        raise ArgumentError if name == nil
        define_method(name, &block) 
      end
  end

  # This method generates a new Class, if no parent is supplied it will extend Base
  def self.generate_new_class(name, parent=nil)
    if(parent != nil)
      const_set(name, Class.new(Flex.const_get(parent)))
    else
      const_set(name, Class.new(Base))
    end
  end
end