# This class is in charge of parsing the AutomationGenricEnv file, and generate
# class of the Flex display objects
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.

require 'rexml/document'
require 'flex'

class XmlParser
  
  private_class_method  :new
  
  # Parses the xml file stated in Conf
  def self.parse
    path = File.dirname(File.expand_path(__FILE__))
    parse_file(path + "/" + $automation_environment)
  end
  
  # Parses the xml file provided
  # It generates classes and methods based on the file
  def self.parse_file(document) 
    file = File.new(document)
    doc = REXML::Document.new file
    doc.elements.each("TypeInformation/ClassInfo") do |element|
      class_name = element.attributes["Name"]

      #-- Generates a new class
      new_class = Flex.generate_new_class(class_name, element.attributes["Extends"])

      #-- If the component supports tabular data, methods are created
      if(element.attributes["SupportsTabularData"] == "true")
        add_tabular_method(new_class)
      end
    
      #-- For all events a relating method is created
      events = REXML::Document.new element.to_s
      events.elements.each("ClassInfo/Events/Event") do |event|
        add_event(new_class, event)
      end
    end
  end
  
  # This method adds methods for getting and handling tabular data values
  # of a display object. It provides the possibility to, 
  # make the framework wait for a defined amount of seconds if the dataprovider
  # is slow.
  def self.add_tabular_method(class_name)
    class_name.add_method("tabular_data") do
      |*argv|
      options = argv.shift || {}
      
      #-- If a wait tag is defined, the framework will loop until it has gotten the
      # information or the time is up.
      if(options[:wait] != nil)
        count = options[:wait]

        data = flex_object.getTabularData(@id, options[:start], options[:end])
        while(data != nil && count > 0)
          sleep(1)
          count = count - 1
          data = flex_object.getTabularData(@id, options[:start], options[:end])
        end
      else
        data = flex_object.getTabularData(@id, options[:start], options[:end])
      end
      
      return data
    end
    
    # This method extracts singular values
    class_name.add_method("tabular_data_value") do
      |arg|
      flex_object.getTabularDataValue(@id, arg)
    end
    
    # This method returns the index of the first visible row
    class_name.add_method("first_visible_row") do
      flex_object.firstVisibleRow(@id)
    end
    
    # This method returns the index of the last visible row
    class_name.add_method("last_visible_row") do
      flex_object.lastVisibleRow(@id)
    end
    
    # This method returns the number of rows
    class_name.add_method("num_rows") do
      flex_object.numRows(@id)
    end
    
    # This method returns the number of columns
    class_name.add_method("num_columns") do
      flex_object.numColumns(@id)
    end
    
    # This method returns the column names
    class_name.add_method("column_names") do
      flex_object.columnNames(@id)
    end
    
  end
  
  # This method adds a method to a class that is corresponding to the FLex event of the Flex component
  def self.add_event(class_name,event)
    
    event_class = event[1].attributes["Class"]
    event_type = event[1].attributes["Type"]
    method = event.attributes["Name"]

    #-- Adds a method to the Flex object, which is the name of an event in ActionScript
    # The developer can override the default tags, and they are braided in.
    class_name.add_method(shift_case(method)) do
      |*argv|
      options = argv.shift || {}
      
      properties = REXML::Document.new event.to_s
      
      options.each_pair do |key, value|
        properties.elements.each("Event/Property") do |property|
          if(property.attributes["Name"] == shift_case(key.to_s, true))
            property.attributes["DefaultValue"] = value
          end
        end
      end
      
 
      # This part handles synchronization between a Flex object and the framework.
      # The developer needs to add a wait tag to the event.
      # It loops over and tries the object until it gets a hit or that the time
      # runs out.

      if(options[:wait] != nil)
        count = options[:wait]

        new_event = flex_object.addevent(event_type, event_class, @id, properties.to_s)
        while((new_event != nil || new_event == "Not synchronized")&& count > 0)
          sleep(1)
          count = count - 1
          new_event = flex_object.addevent(event_type, event_class, @id, properties.to_s)
        end
      else
        new_event = flex_object.addevent(event_type, event_class, @id, properties.to_s)
        if(new_event == "Not synchronized")
          count = $wait_sync
          while(new_event == "Not synchronized" && count > 0)
            sleep(1)
            count = count -1
            new_event = flex_object.addevent(event_type, event_class, @id, properties.to_s)
          end
        end
      end

      raise new_event unless new_event == nil
      
      is_sync
      
      sleep(funfx.speed) # To make the interacttion slow enough so people can watch
    end
  end
  
  # This method makes the methods to correspond to Ruby conventions with lower case letters
  def self.shift_case(str)
      str.gsub(/^[A-Z]|[A-Z]/) { |a| "_"  + a.downcase}.gsub(/^[_]/){}
  end
  
end





  


