require 'test/unit'
require 'flexmock'
require 'lib/funfx'
require 'lib/xml_parser'

# This test case tests everything concerning the Base class
#--
# Copyright (c) 2007 Peter Nicolai Motzfeldt.
# Licensed under the same terms as Ruby. No warranty is provided.
class BaseTest < Test::Unit::TestCase
  include FlexMock::TestCase
  
  def setup
    @parentName = "FlexObject"
    @childName = "FlexButton"
    @id = "tempid"
  end
  
  # This test tests the generation of new classes
  def test_generate_class
    assert_raise(NameError){
      Flex::TestClass
    }
    Flex.generate_new_class("TestClass")
    assert_not_nil(Flex::TestClass)
    
    assert_raise(NameError){
      Flex::TestClassChild
    }
    Flex.generate_new_class("TestClassChild", "TestClass")
    assert_not_nil(Flex::TestClassChild)
    assert_equal(Flex::TestClassChild.superclass, Flex::TestClass)
  end
  
  # This tests tests the parsing of the xml file, and generates the class hierarchy
  def test_a_parse
    assert_raise(NameError){
      Flex::FlexDisplayObject
    }
    Funfx.instance #Builds all classes
    assert_not_nil(Flex::DisplayObject)
  end
  
  # This test tests the creation of instances of the classes
  def test_b_create
    parent = Flex::Base.new(@parentName, @id)
    assert_not_nil(parent)
    assert_equal(@parentName, parent.name)
    assert_equal(@id, parent.id)
    
    child = Flex::Base.new(@childName, "id")
    assert_not_nil(child)
    assert_equal(@childName, child.name)
    assert_equal("id", child.id)
  end
  
  # This test tests that a nil class raises an exception
  def test_c_create_nil
    assert_raise(ArgumentError){
      parent = Flex::Base.new(nil)
    }
  end
  
  # This method tests to add a method to a created class
  def test_d_add_method
     Flex::Base.add_method("newMethod"){ |arg| return arg }
     parent = Flex::Base.new(@parentName, @id)
     assert_equal(true, parent.newMethod(true))
     assert_equal(8, parent.newMethod(8))
  end
  
  # This method tests to get a child of a a created object
  def test_e_new_object 
    mockBase = Flex::MockBase.new(@parentName,@id)
    assert_not_nil(mockBase.button(@childName))
  end
  
  # This test tests taht if no child is found it returns nil
  def test_f_new_object_nil
    mockBase = Flex::MockBase.new(@parentName, @id)
    assert_nil(mockBase.button(""))
  end
  
  # This test tests the to get the property of a object
  def test_g_check_property
    mockBase = Flex::MockBase.new(@parentName, @id)
    assert_not_nil(mockBase.text)
    assert_equal("Test", mockBase.text)
  end
  
  # This test tests that it raises an error if a methos which does not exist is called
  def test_h_check_property_nil
    mockBase = Flex::MockBase.new(@parentName, @id)
    assert_raise(RuntimeError){
      mockBase.bla
    }
  end
  
  # This test tests the tabular data method. It uses a mock of a datagrid
  def test_i_tabular_data
    Funfx.instance
    #mock = Flex.generate_new_class("MockDataGrid","DataGrid")
    mock = Class.new Flex::DataGrid do
      include FlexMock::TestCase
      
      def is_sync
        true
      end
      
      #-- This is a mock method, returning a mock object that expects certain values.
      def flex_object
        m = flexmock("funfx")
        m.should_receive(:getTabularData).with("tempid", nil, nil).and_return("tabularvalue1,tabularvalue2")
        m.should_receive(:getTabularData).with("tempid", 0, 0).and_return("tabularvalue1")
        m.should_receive(:getTabularData).with("tempid", 0, nil).and_return("tabularvalue1")
        m.should_receive(:getTabularData).with("tempid", 1, 1).and_return("")
        return m
      end
    end
    
    
    begin
      Flex.const_get("MockDataGrid")
    rescue
      Object.const_set "MockDataGrid", mock
    end

    datagrid = MockDataGrid.new(@parentName, @id)
    assert_not_nil(datagrid)
    assert_equal("tabularvalue1,tabularvalue2",datagrid.tabular_data)
    assert_equal("tabularvalue1", datagrid.tabular_data(:start => 0, :end => 0))
    assert_equal("tabularvalue1", datagrid.tabular_data(:start => 0))
    assert_equal("", datagrid.tabular_data(:start => 1, :end => 1))
  end
  
  def test_j_tabular_data_value
    Funfx.instance
    #mock = Flex.generate_new_class("MockDataGrid","DataGrid")
    mock = Class.new Flex::DataGrid do
      include FlexMock::TestCase
      
      def is_sync
        true
      end
      
      def flex_object
        m = flexmock("funfx")
        m.should_receive(:getTabularDataValue).with("tempid", nil, nil).and_return("tabularvalue1,tabularvalue2")
        m.should_receive(:getTabularDataValue).with("tempid", 0, 0).and_return("tabularvalue1")
        m.should_receive(:getTabularData).with("tempid", 0, nil).and_return("tabularvalue1")
        m.should_receive(:getTabularData).with("tempid", 1, 1).and_return("")
        return m
      end
    end
    
    begin
      Flex.const_get("MockDataGrid")
    rescue
      Object.const_set "MockDataGrid", mock
    end

    datagrid = MockDataGrid.new(@parentName, @id)
    assert_not_nil(datagrid)
    assert_equal("tabularvalue1,tabularvalue2",datagrid.tabular_data)
    assert_equal("tabularvalue1", datagrid.tabular_data(:start => 0, :end => 0))
    assert_equal("tabularvalue1", datagrid.tabular_data(:start => 0))
    assert_equal("", datagrid.tabular_data(:start => 1, :end => 1))
  end
   
end

module Flex
  
  # This class is a mock of the base class.
  #--
  # Copyright (c) 2007 Peter Nicolai Motzfeldt.
  # Licensed under the same terms as Ruby. No warranty is provided.
  class MockBase < Flex::Base
    include FlexMock::TestCase
    
    def ready
      return true
    end
    
    #-- This is a mock method, returning a mock object that expects certain values.
    def flex_object
      m = flexmock("funfx")
      m.should_receive(:checkProperty).with(@id,"text").and_return("Test")
      m.should_receive(:checkProperty).with(@id,"bla").and_return(nil)
      m.should_receive(:checkComponent).with("tempid", "FlexObject").and_return("datagrid_id")
      m.should_receive(:checkComponent).with("tempid", "FlexButton").and_return("new_id")
      m.should_receive(:checkComponent).with("tempid", "").and_return(false)
      m.should_receive(:getTabularData).with("tempid").and_return("bla")
      m.should_receive(:ready).with("FUNFX").and_return(true)
      return m
    end
    
    def is_sync
      return true
    end
  
  end
end
  
